/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 21 decembre 2006
   derniere modification le 20/03/2008 */

#ifndef SCECAMERA_H
#define SCECAMERA_H

#include <SCE/interface/SCEFrustum.h>

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct sce_sviewport SCE_SViewport;
struct sce_sviewport
{
    int x, y;
    int w, h;
}; 


/* structure d'une camera */
typedef struct sce_scamera SCE_SCamera;
struct sce_scamera
{
    SCE_TMatrix4 view;      /* matrice de projection */
    SCE_TMatrix4 proj;      /* matrice de projection */
    SCE_SViewport viewport; /* viewport de la camera */
    SCE_SFrustum frustum;   /* frustum de vue de la camera */
};


/* initialise une camera */
void SCE_Camera_Init (SCE_SCamera*);

/* cree une camera */
SCE_SCamera* SCE_Camera_Create (void);
/* supprime une camera */
void SCE_Camera_Delete (SCE_SCamera*);

/* defini la zone de l'ecran ou la camera sera representee */
void SCE_Camera_SetViewport (SCE_SCamera*, int, int, int, int);

/* retourne les matrices d'une camera */
float* SCE_Camera_GetView (SCE_SCamera*);
float* SCE_Camera_GetProj (SCE_SCamera*);

/* retourne le frustum d'une camera */
SCE_SFrustum* SCE_Camera_GetFrustum (SCE_SCamera*);
/* met a jour le frustum d'une camera a partir de ses matrices */
void SCE_Camera_UpdateFrustum (SCE_SCamera*);

/* defini la camera comme camera active */
void SCE_Camera_Use (SCE_SCamera*);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
